@@ -205,7 +205,7 @@ static struct pppcp_packet *pppcp_packet_new(struct pppcp_data *data,
 	guint16 packet_length = bufferlen + sizeof(*packet);
 
 	ppp_packet = ppp_packet_new(packet_length, data->driver->proto);
-	if (!ppp_packet)
+	if (ppp_packet == NULL)
 		return NULL;
 
 	/* advance past protocol to add CP header information */
@@ -310,9 +310,9 @@ static gboolean is_first_request(struct pppcp_timer_data *timer_data)
 
 /* actions */
 /* log an illegal event, but otherwise do nothing */
-static void pppcp_illegal_event(guint8 state, guint8 type)
+static void pppcp_illegal_event(GAtPPP *ppp, guint8 state, guint8 type)
 {
-	g_printerr("Illegal event %d while in state %d\n", type, state);
+	DBG(ppp, "Illegal event %d while in state %d", type, state);
 }
 
 static void pppcp_this_layer_up(struct pppcp_data *data)
@@ -690,7 +690,7 @@ static void pppcp_generate_event(struct pppcp_data *data,
 	return;
 
 error:
-	pppcp_illegal_event(data->state, event_type);
+	pppcp_illegal_event(data->ppp, data->state, event_type);
 }
 
 void pppcp_signal_open(struct pppcp_data *data)
@@ -1002,7 +1002,7 @@ struct pppcp_data *pppcp_new(GAtPPP *ppp, const struct pppcp_proto *proto,
 	struct pppcp_data *data;
 
 	data = g_try_malloc0(sizeof(struct pppcp_data));
-	if (!data)
+	if (data == NULL)
 		return NULL;
 
 	if (dormant)
