@@ -58,6 +58,7 @@
 #include <ofono/voicecall.h>
 #include <ofono/gprs.h>
 #include <ofono/gprs-context.h>
+#include <ofono/gnss.h>
 
 #include <drivers/atmodem/vendor.h>
 #include <drivers/atmodem/sim-poll.h>
@@ -77,19 +78,36 @@ struct phonesim_data {
 struct gprs_context_data {
 	GAtChat *chat;
 	char *interface;
+	enum ofono_gprs_proto proto;
 };
 
 static void at_cgact_up_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct cb_data *cbd = user_data;
-	ofono_gprs_context_up_cb_t cb = cbd->cb;
+	ofono_gprs_context_cb_t cb = cbd->cb;
 	struct ofono_gprs_context *gc = cbd->user;
 	struct gprs_context_data *gcd = ofono_gprs_context_get_data(gc);
 	struct ofono_error error;
 
 	decode_at_error(&error, g_at_result_final_response(result));
-	cb(&error, ok ? gcd->interface : NULL, FALSE,
-			NULL, NULL, NULL, NULL, cbd->data);
+
+	if (ok == FALSE)
+		goto done;
+
+	ofono_gprs_context_set_interface(gc, gcd->interface);
+
+	if (gcd->proto == OFONO_GPRS_PROTO_IP ||
+			gcd->proto == OFONO_GPRS_PROTO_IPV4V6)
+		ofono_gprs_context_set_ipv4_address(gc, NULL, FALSE);
+
+	if (gcd->proto == OFONO_GPRS_PROTO_IPV6 ||
+			gcd->proto == OFONO_GPRS_PROTO_IPV4V6) {
+		ofono_gprs_context_set_ipv6_address(gc, "fe80::1");
+		ofono_gprs_context_set_ipv6_prefix_length(gc, 10);
+	}
+
+done:
+	cb(&error, cbd->data);
 }
 
 static void at_cgact_down_cb(gboolean ok, GAtResult *result, gpointer user_data)
@@ -104,16 +122,32 @@ static void at_cgact_down_cb(gboolean ok, GAtResult *result, gpointer user_data)
 
 static void phonesim_activate_primary(struct ofono_gprs_context *gc,
 				const struct ofono_gprs_primary_context *ctx,
-				ofono_gprs_context_up_cb_t cb, void *data)
+				ofono_gprs_context_cb_t cb, void *data)
 {
 	struct gprs_context_data *gcd = ofono_gprs_context_get_data(gc);
 	struct cb_data *cbd = cb_data_new(cb, data);
 	char buf[OFONO_GPRS_MAX_APN_LENGTH + 128];
-	int len;
+	int len = 0;
 
 	cbd->user = gc;
+	gcd->proto = ctx->proto;
 
-	len = snprintf(buf, sizeof(buf), "AT+CGDCONT=%u,\"IP\"", ctx->cid);
+	switch (ctx->proto) {
+	case OFONO_GPRS_PROTO_IP:
+		len = snprintf(buf, sizeof(buf), "AT+CGDCONT=%u,\"IP\"",
+				ctx->cid);
+		break;
+
+	case OFONO_GPRS_PROTO_IPV6:
+		len = snprintf(buf, sizeof(buf), "AT+CGDCONT=%u,\"IPV6\"",
+				ctx->cid);
+		break;
+
+	case OFONO_GPRS_PROTO_IPV4V6:
+		len = snprintf(buf, sizeof(buf), "AT+CGDCONT=%u,\"IPV4V6\"",
+				ctx->cid);
+		break;
+	}
 
 	if (ctx->apn)
 		snprintf(buf + len, sizeof(buf) - len - 3, ",\"%s\"",
@@ -131,7 +165,7 @@ static void phonesim_activate_primary(struct ofono_gprs_context *gc,
 error:
 	g_free(cbd);
 
-	CALLBACK_WITH_FAILURE(cb, NULL, 0, NULL, NULL, NULL, NULL, data);
+	CALLBACK_WITH_FAILURE(cb, data);
 }
 
 static void phonesim_deactivate_primary(struct ofono_gprs_context *gc,
@@ -669,6 +703,7 @@ static void phonesim_post_online(struct ofono_modem *modem)
 	if (mw)
 		ofono_message_waiting_register(mw);
 
+	ofono_gnss_create(modem, 0, "atmodem", data->chat);
 }
 
 static struct ofono_modem_driver phonesim_driver = {
